@@ -8,7 +8,7 @@ from typing_extensions import Literal, NotRequired, Protocol, TypedDict, TypeVar
 from .hybrid_observer import HybridObservable
 
 # Literals
-FrameType = Literal["video", "motion"]
+FrameType = Literal["stream", "motion"]
 CameraType = Literal["camera", "doorbell"]
 ZoneType = Literal["intersect", "contain"]
 ZoneFilter = Literal["include", "exclude"]
@@ -51,6 +51,10 @@ CameraPublicProperties = Literal[
     "hasLight",
     "hasSiren",
     "hasBinarySensor",
+    "hasMotionDetection",
+    "hasObjectDetection",
+    "hasAudioDetection",
+    "hasPtz",
     "hasBattery",
     "info",
     "type",
@@ -67,12 +71,15 @@ DeviceManagerEventType = Literal[
     "cameraSelected",
     "cameraDeselected",
 ]
-JsonSchemaType = Literal["string", "number", "boolean", "object", "array", "button"]
+JsonSchemaType = Literal["string", "number", "boolean", "array", "button", "submit"]
 LoggerLevel = Literal["error", "warn", "info", "debug", "trace", "attention", "success"]
 APIEventType = Literal["finishLaunching", "shutdown"]
 PTZMovementType = Literal["absolute", "relative", "continuous", "home", "preset", "stop"]
 RTSPAudioCodec = Literal["aac", "opus", "pcma"]
 ProbeAudioCodec = Literal["aac", "opus", "pcma"]
+HwAccelMethod = Literal[
+    "auto", "cuda", "vaapi", "videotoolbox", "qsv", "rkmpp", "v4l2m2m", "opencl", "vulkan", "amf", "jetson"
+]
 
 # Basic types
 Callback = Union[
@@ -380,7 +387,7 @@ class FrameMetadata(TypedDict):
 class ImageInformation(TypedDict):
     width: int
     height: int
-    channels: Literal[1, 3, 4]
+    channels: float
     format: ImageInputFormat
 
 
@@ -458,7 +465,7 @@ class CameraConfigInputSettings(TypedDict):
     role: CameraRole
     useForSnapshot: bool
     hotMode: bool
-    urls: list[str]
+    urls: NotRequired[list[str]]
 
 
 class BaseCameraConfig(TypedDict):
@@ -468,6 +475,10 @@ class BaseCameraConfig(TypedDict):
     hasLight: Optional[bool]
     hasSiren: Optional[bool]
     hasBinarySensor: Optional[bool]
+    hasMotionDetection: Optional[bool]
+    hasObjectDetection: Optional[bool]
+    hasAudioDetection: Optional[bool]
+    hasPtz: Optional[bool]
     hasBattery: Optional[bool]
     disabled: Optional[bool]
     info: Optional[CameraInformation]
@@ -477,8 +488,13 @@ class CameraConfig(BaseCameraConfig):
     sources: list[CameraConfigInputSettings]
 
 
+class PartialCameraConfig(BaseCameraConfig, total=False):
+    sources: NotRequired[list[CameraConfigInputSettings]]
+
+
 class CameraDelegate(Protocol):
     async def snapshot(self, source_id: str, force_new: Optional[bool] = None) -> Optional[bytes]: ...
+    async def streamUrl(self, source_name: str) -> str: ...
 
 
 class PTZSpeed(TypedDict):
@@ -628,6 +644,10 @@ class Camera(TypedDict):
     hasLight: bool
     hasSiren: bool
     hasBinarySensor: bool
+    hasMotionDetection: bool
+    hasObjectDetection: bool
+    hasAudioDetection: bool
+    hasPtz: bool
     hasBattery: bool
     info: CameraInformation
     type: CameraType
@@ -733,6 +753,8 @@ class CameraDevice(Protocol):
     @property
     def ptz(self) -> CameraPTZDelegate: ...
     @property
+    def delegate(self) -> CameraDelegate: ...
+    @property
     def sources(self) -> list[CameraDeviceSource]: ...
 
     on_connected: HybridObservable[bool]
@@ -868,12 +890,6 @@ class CameraDevice(Protocol):
         event_data: BatterySetEvent,
     ) -> None: ...
 
-    async def add_camera_source(self, source: CameraConfigInputSettings) -> None: ...
-    async def update_camera_source(
-        self, source_id: str, source: CameraConfigInputSettingsPartial
-    ) -> None: ...
-    async def remove_camera_source(self, source_id: str) -> None: ...
-
 
 CameraSelectedCallback = Union[
     Callable[[CameraDevice, CameraExtension], None],
@@ -888,10 +904,11 @@ CameraDeselectedCallback = Union[
 @runtime_checkable
 class DeviceManager(Protocol):
     async def create_camera(self, camera_config: CameraConfig) -> CameraDevice: ...
-    async def get_camera_by_name(self, camera_name: str) -> Optional[CameraDevice]: ...
-    async def get_camera_by_id(self, camera_id: str) -> Optional[CameraDevice]: ...
-    async def remove_camera_by_name(self, camera_name: str) -> None: ...
-    async def remove_camera_by_id(self, camera_id: str) -> None: ...
+    async def update_camera(
+        self, camera_id_or_name: str, camera_config: PartialCameraConfig
+    ) -> CameraDevice: ...
+    async def get_camera(self, camera_id_or_name: str) -> Optional[CameraDevice]: ...
+    async def remove_camera(self, camera_id_or_name: str) -> None: ...
 
     @overload
     def on(self, event: Literal["cameraSelected"], f: CameraSelectedCallback) -> Any: ...
@@ -915,7 +932,7 @@ class DeviceManager(Protocol):
 class CoreManager(Protocol):
     async def connect_to_plugin(self, plugin_name: str) -> "Optional[PluginInterface]": ...
     async def get_ffmpeg_path(self) -> str: ...
-    async def get_hwaccel_info(self, options: Optional["HWAccelOptions"] = None) -> "FfmpegArgs": ...
+    async def get_hwaccel_info(self, options: "HWAccelOptions") -> "list[FfmpegArgs]": ...
     async def get_server_addresses(self) -> list[str]: ...
     async def get_ice_servers(self) -> list["IceServer"]: ...
 
@@ -925,20 +942,19 @@ class HWAccelScale(TypedDict):
     height: int
 
 
-class HWAccelOptions(TypedDict, total=False):
+class HWAccelOptions(TypedDict):
     targetCodec: Union[Literal["h264"], Literal["h265"]]
-    keepOnHardware: bool
-    pixelFormat: str
-    scale: HWAccelScale
+    keepOnHardware: NotRequired[Optional[bool]]
+    pixelFormat: NotRequired[Optional[str]]
+    scale: NotRequired[Optional[HWAccelScale]]
 
 
 class FfmpegArgs(TypedDict):
     codec: str
-    hwaccel: str
+    hwaccel: HwAccelMethod
     hwaccelArgs: list[str]
     hwaccelFilters: list[str]
     hwDeviceArgs: list[str]
-    threads: str
     supported: bool
 
 
@@ -951,29 +967,33 @@ class IceServer(TypedDict):
 # Schema related types and interfaces
 PluginConfig = dict[str, Any]
 
-J = TypeVar("J")
+J = TypeVar("J", bound=Union[str, list[str], int, list[int], bool, list[bool], Any])
 
 
-class JsonBaseSchema(TypedDict, Generic[J]):
-    key: NotRequired[str]
-    title: NotRequired[str]
-    description: NotRequired[str]
+class JsonFactorySchema(TypedDict):
+    key: str
+    title: str
+    description: str
+    group: NotRequired[str]
+
+
+class JsonBaseSchmaWithoutCallbacks(JsonFactorySchema, Generic[J]):
+    hidden: NotRequired[bool]
     required: NotRequired[bool]
     readonly: NotRequired[bool]
     placeholder: NotRequired[str]
-    hidden: NotRequired[bool]
-    group: NotRequired[str]
     defaultValue: NotRequired[J]
+
+
+class JsonBaseSchema(JsonBaseSchmaWithoutCallbacks[J], Generic[J]):
     store: NotRequired[bool]
     onSet: NotRequired[
         Union[Callable[[Any, Any], Awaitable[Union[None, Any]]], Callable[[Any, Any], Union[None, Any]]]
-    ]  # only for plugins
-    onGet: NotRequired[
-        Union[Callable[[], Awaitable[Union[Any, None]]], Callable[[], Union[Any, None]]]
-    ]  # only for plugins
+    ]
+    onGet: NotRequired[Union[Callable[[], Awaitable[Union[Any, None]]], Callable[[], Union[Any, None]]]]
 
 
-class JsonSchemaString(JsonBaseSchema[str]):
+class JsonStringSchema(TypedDict):
     type: Literal["string"]
     format: NotRequired[
         Literal["date-time", "date", "time", "email", "uuid", "ipv4", "ipv6", "password", "qrCode", "image"]
@@ -982,48 +1002,153 @@ class JsonSchemaString(JsonBaseSchema[str]):
     maxLength: NotRequired[int]
 
 
-class JsonSchemaNumber(JsonBaseSchema[Union[int, float]]):
+class JsonNumberSchema(TypedDict):
     type: Literal["number"]
-    minimum: NotRequired[float]
-    maximum: NotRequired[float]
-    step: NotRequired[float]
+    minimum: NotRequired[int]
+    maximum: NotRequired[int]
+    step: NotRequired[int]
 
 
-class JsonSchemaBoolean(JsonBaseSchema[bool]):
+class JsonBooleanSchema(TypedDict):
     type: Literal["boolean"]
 
 
-class JsonSchemaEnum(JsonBaseSchema[Union[str, list[str]]]):
+class JsonEnumSchema(TypedDict):
     type: Literal["string"]
     enum: list[str]
     multiple: NotRequired[bool]
 
 
-class JsonSchemaObject(JsonBaseSchema[dict[Any, Any]]):
-    type: Literal["object"]
-    opened: NotRequired[bool]
-    properties: NotRequired["JsonSchemaForm"]
-
-
-class JsonSchemaArray(JsonBaseSchema[list[Any]]):
+class JsonArraySchema(TypedDict):
     type: Literal["array"]
     opened: NotRequired[bool]
-    items: NotRequired["JsonSchema"]
+    items: "JsonSchemaWithoutCallbacksAndKey"
 
 
-class JsonSchemaButton(JsonBaseSchema[Any]):
+class JsonSchemaString(JsonBaseSchema[str], JsonStringSchema):
+    pass
+
+
+class JsonSchemaStringWithoutCallbacks(JsonBaseSchmaWithoutCallbacks[str], JsonStringSchema):
+    pass
+
+
+class JsonSchemaNumber(JsonBaseSchema[int], JsonNumberSchema):
+    pass
+
+
+class JsonSchemaNumberWithoutCallbacks(JsonBaseSchmaWithoutCallbacks[int], JsonNumberSchema):
+    pass
+
+
+class JsonSchemaBoolean(JsonBaseSchema[bool], JsonBooleanSchema):
+    pass
+
+
+class JsonSchemaBooleanWithoutCallbacks(JsonBaseSchmaWithoutCallbacks[bool], JsonBooleanSchema):
+    pass
+
+
+class JsonSchemaEnum(JsonBaseSchema[Union[str, list[str]]], JsonEnumSchema):
+    pass
+
+
+class JsonSchemaEnumWithoutCallbacks(JsonBaseSchmaWithoutCallbacks[Union[str, list[str]]], JsonEnumSchema):
+    pass
+
+
+class JsonSchemaArray(JsonBaseSchema[Union[list[str], list[int], list[bool]]], JsonArraySchema):
+    pass
+
+
+class JsonSchemaArrayWithoutCallbacks(
+    JsonBaseSchmaWithoutCallbacks[Union[list[str], list[int], list[bool]]], JsonArraySchema
+):
+    pass
+
+
+class JsonSchemaButton(JsonFactorySchema):
     type: Literal["button"]
     color: NotRequired[Literal["success", "info", "warn", "danger"]]
+    onSet: Callable[[], Awaitable[None]]
 
 
-class JsonSchemaObjectButton(TypedDict):
-    label: str
-    onSubmit: str
+class JsonSchemaSubmit(JsonFactorySchema):
+    type: Literal["submit"]
     color: NotRequired[Literal["success", "info", "warn", "danger"]]
+    onClick: Union[Callable[["FormSubmitResponse"], None], Awaitable[Union["FormSubmitResponse", None]]]
 
 
-class JsonSchemaObjectWithButtons(JsonSchemaObject):
-    buttons: list[JsonSchemaObjectButton]
+class JsonSchemaStringWithoutCallbacksAndKey(TypedDict):
+    type: Literal["string"]
+    title: str
+    description: str
+    group: NotRequired[str]
+    hidden: NotRequired[bool]
+    required: NotRequired[bool]
+    readonly: NotRequired[bool]
+    placeholder: NotRequired[str]
+    defaultValue: NotRequired[str]
+    format: NotRequired[
+        Literal["date-time", "date", "time", "email", "uuid", "ipv4", "ipv6", "password", "qrCode", "image"]
+    ]
+    minLength: NotRequired[int]
+    maxLength: NotRequired[int]
+
+
+class JsonSchemaNumberWithoutCallbacksAndKey(TypedDict):
+    type: Literal["number"]
+    title: str
+    description: str
+    group: NotRequired[str]
+    hidden: NotRequired[bool]
+    required: NotRequired[bool]
+    readonly: NotRequired[bool]
+    placeholder: NotRequired[str]
+    defaultValue: NotRequired[int]
+    minimum: NotRequired[int]
+    maximum: NotRequired[int]
+    step: NotRequired[int]
+
+
+class JsonSchemaBooleanWithoutCallbacksAndKey(TypedDict):
+    type: Literal["boolean"]
+    title: str
+    description: str
+    group: NotRequired[str]
+    hidden: NotRequired[bool]
+    required: NotRequired[bool]
+    readonly: NotRequired[bool]
+    placeholder: NotRequired[str]
+    defaultValue: NotRequired[bool]
+
+
+class JsonSchemaEnumWithoutCallbacksAndKey(TypedDict):
+    type: Literal["string"]
+    title: str
+    description: str
+    group: NotRequired[str]
+    hidden: NotRequired[bool]
+    required: NotRequired[bool]
+    readonly: NotRequired[bool]
+    placeholder: NotRequired[str]
+    defaultValue: NotRequired[Union[str, list[str]]]
+    enum: list[str]
+    multiple: NotRequired[bool]
+
+
+class JsonSchemaArrayWithoutCallbacksAndKey(TypedDict):
+    type: Literal["array"]
+    title: str
+    description: str
+    group: NotRequired[str]
+    hidden: NotRequired[bool]
+    required: NotRequired[bool]
+    readonly: NotRequired[bool]
+    placeholder: NotRequired[str]
+    defaultValue: NotRequired[Union[list[str], list[int], list[bool]]]
+    opened: NotRequired[bool]
+    items: "JsonSchemaWithoutCallbacksAndKey"
 
 
 JsonSchema = Union[
@@ -1031,17 +1156,26 @@ JsonSchema = Union[
     JsonSchemaNumber,
     JsonSchemaBoolean,
     JsonSchemaEnum,
-    JsonSchemaObject,
-    JsonSchemaObjectWithButtons,
     JsonSchemaArray,
     JsonSchemaButton,
+    JsonSchemaSubmit,
 ]
 
-JsonSchemaForm = dict[str, JsonSchema]
+JsonSchemaWithoutCallbacks = Union[
+    JsonSchemaStringWithoutCallbacks,
+    JsonSchemaNumberWithoutCallbacks,
+    JsonSchemaBooleanWithoutCallbacks,
+    JsonSchemaEnumWithoutCallbacks,
+    JsonSchemaArrayWithoutCallbacks,
+]
 
-
-class RootSchema(TypedDict):
-    schema: JsonSchemaForm
+JsonSchemaWithoutCallbacksAndKey = Union[
+    JsonSchemaStringWithoutCallbacksAndKey,
+    JsonSchemaNumberWithoutCallbacksAndKey,
+    JsonSchemaBooleanWithoutCallbacksAndKey,
+    JsonSchemaEnumWithoutCallbacksAndKey,
+    JsonSchemaArrayWithoutCallbacksAndKey,
+]
 
 
 class ToastMessage(TypedDict):
@@ -1050,16 +1184,16 @@ class ToastMessage(TypedDict):
 
 
 class FormSubmitSchema(TypedDict):
-    config: JsonSchemaObjectWithButtons
+    config: dict[str, Any]
 
 
 class FormSubmitResponse(TypedDict, total=False):
-    toast: ToastMessage
-    schema: FormSubmitSchema
+    toast: NotRequired[ToastMessage]
+    schema: NotRequired[list[JsonSchemaWithoutCallbacks]]
 
 
 class SchemaConfig(TypedDict):
-    rootSchema: RootSchema
+    schema: list[JsonSchema]
     config: dict[str, Any]
 
 
@@ -1089,14 +1223,12 @@ class BasePlugin(metaclass=ABCMeta):
     @abstractmethod
     def __init__(self, logger: "LoggerService", api: "PluginAPI") -> None: ...
     @abstractmethod
-    async def onFormSubmit(self, action_id: str, payload: Any) -> Union[FormSubmitResponse, None]: ...
-    @abstractmethod
     async def configureCameras(self, cameras: list[CameraDevice]) -> None: ...
 
 
 class MotionDetectionPlugin(BasePlugin):
     @abstractmethod
-    async def interfaceSchema(self) -> Optional[RootSchema]: ...
+    async def interfaceSchema(self) -> Optional[list[JsonSchema]]: ...
     @abstractmethod
     async def detectMotion(
         self, video_data: bytes, config: dict[str, Any]
@@ -1105,7 +1237,7 @@ class MotionDetectionPlugin(BasePlugin):
 
 class ObjectDetectionPlugin(BasePlugin):
     @abstractmethod
-    async def interfaceSchema(self) -> Optional[RootSchema]: ...
+    async def interfaceSchema(self) -> Optional[list[JsonSchema]]: ...
     @abstractmethod
     async def detectObjects(
         self, image_data: bytes, metadata: ImageMetadata, config: dict[str, Any]
@@ -1114,7 +1246,7 @@ class ObjectDetectionPlugin(BasePlugin):
 
 class AudioDetectionPlugin(BasePlugin):
     @abstractmethod
-    async def interfaceSchema(self) -> Optional[RootSchema]: ...
+    async def interfaceSchema(self) -> Optional[list[JsonSchema]]: ...
     @abstractmethod
     async def detectAudio(
         self, audio_data: bytes, metadata: AudioMetadata, config: dict[str, Any]
@@ -1123,9 +1255,8 @@ class AudioDetectionPlugin(BasePlugin):
 
 @runtime_checkable
 class PluginInterface(Protocol):
-    async def onFormSubmit(self, action_id: str, payload: Any) -> Union[FormSubmitResponse, None]: ...
     async def configureCameras(self, camera_devices: list[CameraDevice]) -> None: ...
-    async def interfaceSchema(self) -> Optional[RootSchema]: ...
+    async def interfaceSchema(self) -> Optional[list[JsonSchema]]: ...
     async def detectMotion(
         self, video_data: bytes, config: dict[str, Any]
     ) -> MotionDetectionPluginResponse: ...
@@ -1148,7 +1279,7 @@ class LoggerService(Protocol):
     def success(self, *args: Any) -> None: ...
 
 
-PA = TypeVar("PA", bound="CameraStorage[Any]", default="CameraStorage[Any]")
+PA = TypeVar("PA", bound="DeviceStorage[Any]", default="DeviceStorage[Any]")
 
 
 @runtime_checkable
@@ -1156,9 +1287,7 @@ class PluginAPI(Protocol):
     core_manager: CoreManager
     device_manager: DeviceManager
     storage_controller: "StorageController"
-    config_service: "ConfigService"
     storage_path: str
-    config_file: str
 
     def on(self, event: APIEventType, f: Callback) -> Any: ...
     def once(self, event: APIEventType, f: Callback) -> Any: ...
@@ -1166,72 +1295,34 @@ class PluginAPI(Protocol):
     def remove_all_listeners(self, event: Optional[APIEventType] = None) -> None: ...
 
 
-@runtime_checkable
-class ConfigService(Protocol):
-    def get(
-        self,
-        key: Path,
-        default_value: Optional[JSONValue] = None,
-        validate: Optional[Callable[[Any], bool]] = None,
-        refresh: bool = False,
-        write_if_not_valid: bool = False,
-    ) -> Any: ...
-    def has(self, key: Path, refresh: bool = False) -> bool: ...
-    def ensure_exists(self, key: Path, default_value: JSONValue, write: bool = False) -> None: ...
-    def set(self, key: Path, value: Any, write: bool = False) -> None: ...
-    def insert(self, key: Path, value: Any, at: int = 0, write: bool = False) -> None: ...
-    def push(self, key: Path, write: bool = False, *items: Any) -> None: ...
-    def delete(self, key: Path, write: bool = False) -> None: ...
-    def all(self, refresh: bool = False) -> dict[str, Any]: ...
-    def replace(self, config: dict[str, Any], write: bool = False) -> None: ...
-    def update_value(
-        self,
-        path: str,
-        search_key: str,
-        search_value: Any,
-        target_key: str,
-        new_value: Any,
-        write: bool = False,
-    ) -> None: ...
-    def replace_or_add_item(
-        self, path: str, search_key: str, search_value: Any, new_item: Any, write: bool = False
-    ) -> None: ...
-
-
 V1 = TypeVar("V1", default=str)
 V2 = TypeVar("V2", default=dict[str, Any])
 
 
 @runtime_checkable
-class CameraStorage(Protocol, Generic[V2]):
+class DeviceStorage(Protocol, Generic[V2]):
     values: V2
-    schema: JsonSchemaForm
+    schemas: list[JsonSchema]
 
     @overload
-    async def getValue(self, path: str) -> Union[V1, None]: ...
+    async def getValue(self, key: str) -> Union[V1, None]: ...
     @overload
-    async def getValue(self, path: str, default_value: V1) -> V1: ...
-    async def getValue(self, path: str, default_value: Optional[V1] = None) -> Union[V1, None]: ...
+    async def getValue(self, key: str, default_value: V1) -> V1: ...
+    async def getValue(self, key: str, default_value: Optional[V1] = None) -> Union[V1, None]: ...
     async def setValue(self, path: str, new_value: Any) -> None: ...
-    def hasValue(self, path: str) -> bool: ...
+    async def submitValue(self, key: str, new_value: Any) -> Union[FormSubmitResponse, None]: ...
+    def hasValue(self, key: str) -> bool: ...
     async def getConfig(self) -> SchemaConfig: ...
     async def setConfig(self, new_config: V2) -> None: ...
-    @overload
-    async def addSchema(self, schema_or_path: JsonSchemaForm) -> None: ...
-    @overload
-    async def addSchema(self, schema_or_path: str, schema: JsonSchema) -> None: ...
-    async def addSchema(
-        self,
-        schema_or_path: Union[JsonSchemaForm, str],
-        schema: Optional[JsonSchema] = None,
-    ) -> None: ...
-    def removeSchema(self, path: str) -> None: ...
-    async def changeSchema(self, path: str, new_schema: dict[str, Any]) -> None: ...
-    def getSchema(self, path: str) -> Optional[JsonSchema]: ...
-    def hasSchema(self, path: str) -> bool: ...
+    async def addSchema(self, schema: JsonSchema) -> None: ...
+    def removeSchema(self, key: str) -> None: ...
+    async def changeSchema(self, key: str, new_schema: dict[str, Any]) -> None: ...
+    def getSchema(self, key: str) -> Optional[JsonSchema]: ...
+    def hasSchema(self, key: str) -> bool: ...
+    def save(self) -> None: ...
 
 
-S = TypeVar("S", default=CameraStorage[Any], covariant=True)
+S = TypeVar("S", default=DeviceStorage[Any], covariant=True)
 
 
 @runtime_checkable
@@ -1240,7 +1331,14 @@ class StorageController(Protocol[S]):
         self,
         instance: Any,
         camera_id: str,
-        schema: Optional[JsonSchemaForm] = None,
+        schemas: Optional[list[JsonSchema]] = None,
+    ) -> S: ...
+    def create_plugin_storage(
+        self,
+        instance: Any,
+        schemas: Optional[list[JsonSchema]] = None,
     ) -> S: ...
     def get_camera_storage(self, camera_id: str) -> Optional[S]: ...
+    def get_plugin_storage(self) -> Optional[S]: ...
     async def remove_camera_storage(self, camera_id: str) -> None: ...
+    async def remove_plugin_storage(self) -> None: ...
